﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

using System.Data.Services.Common;
using Microsoft.Samples.ServiceHosting.StorageClient;

namespace Micro.Models
{

    [DataServiceKey("PartitionKey", "RowKey")]
    public class Channel
    {
        public string PartitionKey { get; set; }
        public string RowKey { get; set; }

        public DateTime DateCreated { get; set; }

        public Channel()
        {
        }

        public Channel(string name)
        {
            this.DateCreated = DateTime.UtcNow;
            this.PartitionKey = name;
            this.RowKey = string.Empty;
        }

        public static void ConfirmBasicChannelsExist()
        {
            var svc = new Models.MicroData();

            var channels = new string[] { "first", "second" };

            foreach (var channelName in channels)
            {
                var q = from c in svc.Channels
                        where c.PartitionKey == channelName
                        select c;

                if (q.FirstOrDefault() == null)
                {
                    var channel = new Channel(channelName);
                    svc.AddObject("Channels", channel);
                    svc.SaveChanges();
                }                
            }
        }
    }

    [DataServiceKey("PartitionKey", "RowKey")]
    public class Message
    {
        //channel name
        public string PartitionKey { get; set; }
        //datetime of creation
        public string RowKey { get; set; }

        public string Text { get; set; }
        public string Author { get; set; }
        public string Channel { get; set; }
        public DateTime DatePosted { get; set; }
        public int Rating { get; set; }

        public Message()
        {
        }

        public Message(string channel, string author, string text)
        {
            this.Text = text;
            this.Author = author;
            this.Channel = channel;
            this.DatePosted = DateTime.UtcNow;
            this.PartitionKey = MakeKey(channel, this.DatePosted); //channel;
            this.RowKey = System.Xml.XmlConvert.ToString(this.DatePosted,  System.Xml.XmlDateTimeSerializationMode.Utc);
            //var a = new { aa = 2 };
            
        }

        public static string MakeKey(string channel, DateTime dt)
        {
            return channel + "@" + dt.ToString("yyyy-MM-dd");
        }
    }

    public class MicroData : TableStorageDataServiceContext
    {
        public IQueryable<Channel> Channels
        {
            get
            {
                return this.CreateQuery<Channel>("Channels");
            }
        }
        public IQueryable<Message> Messages
        {
            get
            {
                return this.CreateQuery<Message>("Messages");
            }
        }
    }
}
